from tkinter.filedialog import askopenfilenames, askdirectory
from tkinter.font import Font
from tkinter import messagebox
from tkinter import *
import webbrowser
from PIL import Image, ImageOps

ERROR_TITLE = "Image Resizer: Error"
ANCHOR = CENTER
EXAMPLE_FILENAME = "example_file"
EXAMPLE_ADJUNCT = "-resized"
EXAMPLE_EXTENTION = "jpg"

def assemble_adjunct_filename(filename, adjunct, extention): # centralizes adjunct system
    return f"{filename}{adjunct}.{extention}"

def resize_files(filepaths, output_directory, desired_width):
    for filepath in filepaths:
        im = Image.open(filepath)
        im = ImageOps.exif_transpose(im) # fixes rotation

        # current dimensions
        current_width, current_height = im.size

        # if input dimensions are higher than current dimensions, throw an error
        if desired_width > current_width: 
            messagebox.showwarning(ERROR_TITLE, "Increasing dimensions of image") # can still work, so not returning after

        # multiply this by a current dimension and it will output the desired dimension
        ratio_desired_to_current = desired_width/current_width
        # apply ratio
        desired_dimensions = ((desired_width, int(current_height * ratio_desired_to_current))) 
        
        # execute resize
        im_resized = im.resize(desired_dimensions, Image.LANCZOS)

        # this ensures only text after the last "." in the filepath is cutoff
        # (so if your file name is "my.file.jpg" isn't renamed it to just "my-resized.jpg")
        old_filename_no_extention = ".".join(filepath.split("/")[-1].split(".")[:-1])

        # user-indicated appendage to attach to files once shrunk
        # (ex: "my-file.jpg" -> "my-file-resized.jpg")
        output_filename_adjunct = adjunct_entry_value.get()

        # extract file extention
        file_extention = filepath.split(".")[-1]

        # construct output path
        path = f"{output_directory}/{old_filename_no_extention}{output_filename_adjunct}.{file_extention}"
        
        print(f"Output to: {path}")

        # save resized image
        im_resized.save(path, quality=quality_scale.get())

def run_resize():
    # if the input width in the GUI isn't an integer, throw an error
    try:
        desired_width = int(width_entry.get().strip())
    except:
        messagebox.showerror(ERROR_TITLE, "Error: Width is not an integer")
        return

    try: 
        filenames = askopenfilenames(title = "Open Images") # ask for input filepaths
        if not filenames: # if there aren't any files selected, stop
            return

        output_directory = askdirectory(title = "Ouput Directory") # ask for output directory
        if not output_directory: # if there isn't a directory selected, stop
            return
            
        resize_files(filenames, output_directory, desired_width)
    except Exception as e: # if there's any error, display it in the GUI
        messagebox.showerror(ERROR_TITLE, e)    

def adjunct_entry_update_callback(event): # called when filename adjunct is updated
    adjunct = adjunct_entry_value.get() # store adjunct input
    output_file_label.config(text=assemble_adjunct_filename(EXAMPLE_FILENAME, adjunct, EXAMPLE_EXTENTION)) # update output file label
    # this doesn't mean the file has to be jpg, it's just an example of a file name after adding the adjunct

top = Tk()
top.geometry("500x500")
top.title("JPG Resizer -- Adam Stone")

resize_button_font = Font(family='Helvetica', size="36")
small_font = Font(family='Hevetica', size="10", weight="bold", slant="italic")
resize_button = Button(top, text ="Resize Image", command=run_resize, font=resize_button_font, bg="green")
adjunct_entry_value = StringVar()
adjunct_entry = Entry(top, textvariable=adjunct_entry_value)
adjunct_entry.insert(END, "-resized") # set default value
output_file_label = Label(top, text=assemble_adjunct_filename(EXAMPLE_FILENAME, EXAMPLE_ADJUNCT, EXAMPLE_EXTENTION))
quality_label = Label(top, text="Quality:")
quality_scale = Scale(top, from_=1, to=100, orient=HORIZONTAL)
quality_scale.set(50)
width_label = Label(top, text="Width (px):")
width_entry = Entry(top)
width_entry.insert(END, "500") # set default value

credit = Label(top, text="Fab Academy 2023 Charlotte Latin Adam Stone", fg="blue", font=small_font)

adjunct_entry.bind("<KeyRelease>", adjunct_entry_update_callback)

# place all elements on GUI
resize_button.place(relx=0.5, rely=0.3, anchor=ANCHOR)
adjunct_entry.place(relx=0.75, rely=0.55, anchor=ANCHOR)
output_file_label.place(relx=0.25, rely=0.55, anchor=ANCHOR)
quality_label.place(relx=0.3, rely=0.65, anchor=ANCHOR)
quality_scale.place(relx=0.5, rely=0.65, anchor=ANCHOR)
width_label.place(relx=0.3, rely=0.75, anchor=ANCHOR)
width_entry.place(relx=0.5, rely=0.75, anchor=ANCHOR)
credit.place(relx=0.5, rely=0.9, anchor=ANCHOR)

if __name__ == "__main__":
    top.mainloop()